// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      ExceptionErrorHandler.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Oct 27 11:35:43 MET DST 2000

#include "ExceptionErrorHandler.h"
#include "SasException.h"
#include <csignal>
#include <csetjmp>

using namespace SasException;

/*
  Protected run of a routine that is external (not C++) code.
  -----------------------------------------------------------------------------
  In here we allow to catch errors in external code, and convert them into
  C++ exceptions, that are thrown.

  External code can not do any unwinding of the stack, so no cleanup is done!
  In other words, memory leaks cannot be prevented!

 */
class ExceptionErrorHandler::Protect
{
public:
  Protect(ExceptionErrorHandler* parent) : 
    _parent(parent), _error(0), _fatal(0) {}

  ~Protect() { reset(); }

  //
  void run(ExternRoutine);

  // catch fatal errors
  void fatal(Code code, const std::string& text);
  
  // catch normal errors
  void error(Code code, const std::string& text);

private:
  // exceptions...
  void reset();
  void throwAgain();
  void jumpBack();

private:
  ExceptionErrorHandler*   _parent;        
  Error*                   _error;
  Fatal*                   _fatal;
  jmp_buf                  _environment;   // recorded state of environment
};

void ExceptionErrorHandler::Protect::run(ExternRoutine routine)
{
  /* true: start point initialised 
     false: we came back from a long jump
   */
  bool ok=(setjmp(_environment) == 0);
  if (ok) routine();
  else throwAgain();
}

void ExceptionErrorHandler::Protect::fatal(Code code, const std::string& text)
{
  reset();
  _fatal=new Fatal(_parent->name(),code,text);
  jumpBack();
}

void ExceptionErrorHandler::Protect::error(Code code, const std::string& text)
{
  reset();
  _error=new Error(_parent->name(),code,text);
  jumpBack();
}

void ExceptionErrorHandler::Protect::jumpBack()
{
  longjmp(_environment, 123);
}

void ExceptionErrorHandler::Protect::throwAgain()
{
  if      (_error) throw *_error;
  else if (_fatal) throw *_fatal;

  // we never come here (supposedly!)
  else             throw Fatal(_parent->name(),
			       "UnresolvedError",
			       "Trapped unknown error");
}

void ExceptionErrorHandler::Protect::reset()
{
  if (_fatal) {delete _fatal; _fatal=0;}
  if (_error) {delete _error; _error=0;}
}

ExceptionErrorHandler::ExceptionErrorHandler()  : SimpleHandler(), _protect(0)
{
}

ExceptionErrorHandler::~ExceptionErrorHandler()
{
  if (_protect) delete _protect;
}

void ExceptionErrorHandler::error(Code code, const std::string& text)
{
  if (_protect) _protect->error(code,text);
  else throw Error(name(),code,text);
}

void ExceptionErrorHandler::fatal(Code code, const std::string& text)
{
  if (_protect) _protect->fatal(code,text);
  else throw Fatal(name(),code,text);
}

/* Protection is only on if we call an external routine (fortran, C, ...).
 */
void ExceptionErrorHandler::tryExternal(ExternRoutine routine)
{
  Protect* old=_protect;
  _protect=new Protect(this);
  _protect->run(routine);
  delete _protect;
  _protect=old;
}
