// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      ExceptionErrorHandler.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Sep 19 12:03:20 MET DST 2000

/** ExceptionErrorHandler
    Example
  #include <ExceptionErrorHandler.h>
  #include <errstr.h>

  ExceptionErrorHandler local;
  ErrorHandler* oldHandler = errHandler.handler(&local);
  try {
    // piece of code, that may bail out
  }
  catch (SasException::Error e)
  {
    // perhaps we can correct for the error
  }
  catch (SasException::Fatal e)
  {
    // panic signal, one should bail out really
  }
  catch (SasException::Exception e) // last SAS resort
  {
    // Sas library threw an exception, but the implementor did not catch it!
    errHandler.handler(oldHandler) // return to default error handler
    errstr << "Exception: "
           << e.message
           << Sas::fatal("SasException")
  }
  catch (...) //
  {
    errHandler.handler(oldHandler) // return to default error handler
    errstr << "Fatal exception caught, please report bug"
           << Sas::fatal("UnknownException")
  }

  errHandler.handler(oldHandler) // return to default error handler
*/


#ifndef ExceptionErrorHandler_h
#define ExceptionErrorHandler_h

#include "SasException.h"
#include "SimpleHandler.h"

class ExceptionErrorHandler : public SimpleHandler
{
public:
  ExceptionErrorHandler();
  virtual ~ExceptionErrorHandler();

  void error(Code code, const std::string& text);

  void fatal(Code code, const std::string& text);

  // try to mimic exceptions:
  void tryExternal(ExternRoutine);

private:
  class Protect;
  Protect*  _protect;
};


// Convenience class that sets the ExceptionErrorHandler in its
// constructor and restores the existing one in its destructor.

class SetExceptionErrorHandler {
public:
    SetExceptionErrorHandler();
    ~SetExceptionErrorHandler();
private:
    ExceptionErrorHandler exception_handler_;
    ErrorHandler * saved_handler_;
};    

#endif
