// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      SuppressHandler.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Mon Nov  6 10:05:12 MET 2000

#include "SuppressHandler.h"
#include <sstream>
#include <cstdlib> // getenv

SuppressHandler::SuppressHandler() : 
  _all(false),_number(-1)
{
  /* check the environment: if set, silence all warnings */
  if(getenv("SAS_SUPPRESS_WARNING") != 0) {
    std::string c;
    std::stringstream ss;
    ss << getenv("SAS_SUPPRESS_WARNING");
    while (ss >> c) ignore(c);
  }
}

SuppressHandler::~SuppressHandler()
{
}


void SuppressHandler::ignore(const Code& c)
{
  int counter=-1;
  {
    std::stringstream ss;
    ss << c;
    //gcc 4.8.2 portability. string conversion checking
    if (!(ss>>counter))
    	counter = -1;

  }
  if      (c.empty() || counter == 0) _all=true;
  else if (counter > 0)               _number=counter;
  else                                _list[c]=1;

}

void SuppressHandler::allow (const Code& c)
{
  int counter=-1;
  {
    std::stringstream ss;
    ss << c;
    //gcc 4.8.2 portability. string conversion checking
    if (!(ss>>counter))
    	counter = -1;
  }
  
  if (c.empty() || counter == 0) {
    _all=false;
    _list.clear();
  }
  else if (counter > 0) _number=counter;
  else                  _list.erase(c);
}

bool SuppressHandler::suppress(const Code& c)
{
  int number= ++_map[c];

  // ignore all?
  if (_all) return true;
  
  // ignore registered code?
  if (_list[c]) return true;

  // ignore after n times?
  if (_number > -1) return ( (number > _number)?true:false );

  // default
  return false;
}

std::string SuppressHandler::statistics()
{
  std::ostringstream ss;
  bool silenced=false;
  std::string s;

  for (std::map<Code,int>::iterator it=_map.begin();it != _map.end();it++) {
    int number=0;
    
    // count the number of times a warning is silenced
    if      (_all)               number=(*it).second;
    else if (_list[(*it).first]) number=(*it).second;
    else if (_number > 0)        number=(*it).second - _number;

    if ( number > 0 ) {
      ss << "\n   warning " << (*it).first
    		  << " silently occurred " << number << " times";
      silenced=true;
    }
  }

  if (silenced) s=ss.str();
  return s;
}

void SuppressHandler::reset()
{
  _map.clear();
}

std::string SuppressHandler::options()
{
  std::ostringstream ss;
  
  if (_all) {
    ss << " -w";
  }
  else {
    // specify all codes that need to be silenced by default
    for (std::map<Code,int>::iterator it=_list.begin();it != _list.end();it++)
      if ((*it).second) ss << " -w " << (*it).first;
    
    // specify # warnings
    if (_number > -1) ss << " -w " << _number;
  }

  return ss.str();
}
