// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      testExceptionErrorHandler.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Sep 19 14:25:58 MET DST 2000

#include <ExceptionErrorHandler.h>
#include <errstr.h>
#include <f90cxx.h>
#include <sstream>
#include <iostream>
#include <cstdlib>

/***
    Private exceptions
***/
struct NonSasException
{
  NonSasException(int i) : _ii(i) {};
  int _ii;
};

namespace SasException {
  class DivideByZero: public Exception {
  public:
    
    DivideByZero(double divNmbr) : divided(divNmbr)
      {
	message = "I failed the operation [";
	std::stringstream ss;
	ss << divided;
	message += ss.str();
	message += "/0]";
      }
    double divided;

  private:
    DivideByZero();
  };
  
};

/***
    Fortran 90 code
***/
F90MODPROCPROTO(void,faulty,fatal_routine)();
F90MODPROCPROTO(void,faulty,error_routine)();
F90MODPROCPROTO(void,faulty,protected_routine)();
F90MODPROCPROTO(void,faulty,unprotected_routine)();

/***
    C++ code that is called by Fortran 90 code
***/
F90PROCDEF(void,protected_cpp)()
{
  ExceptionErrorHandler local;
  ErrorHandler* oldHandler = errHandler.handler(&local);
  errHandler.name("ThisTestProgram");
  try {
    errstr << "C++[protected_cpp()] is called by f90, "
	   << "but it contains an error"
	   << Sas::error("Error");
  }
  catch (SasException::Exception e) {
    std::cerr << "C++[protected_cpp()] caught the following message:\n   "
	 << e.message << std::endl;
  }
  errHandler.handler(oldHandler); // return to default error handler
}

F90PROCDEF(void,unprotected_cpp)()
{
  errstr << "C++[unprotected_cpp()] This is called by f90, "
	 << "but it contains an error"
	 << Sas::error("Error");
}

/***
    handle all sorts of exceptions
 ***/
void handle(int method)
{
  ErrorHandler* eh=errHandler.handler();
  switch (method) {
  case 0: // thow any exception, but catch locally
    try         {throw NonSasException(123);}
    catch (...) {std::cerr << "Caught exception locally!" << std::endl;}
    break;
  case 1: // thow any exception
    throw NonSasException(123);
    break;
  case 2: // Implicitly throw an Error
    errstr << "An error occured" << Sas::error("ErrorLabel");
    break;
  case 3: // Implicitly throw an Fatal
    errstr << "A fatal error occured" << Sas::fatal("FatalLabel");
    break;
  case 4: // throw this to catch a SasException::Exception
    throw SasException::DivideByZero(123.);
    break;
  case 5: // ...
    eh->tryExternal(F90MODPROC(faulty,error_routine));
    break;
  case 6: // ...
    eh->tryExternal(F90MODPROC(faulty,fatal_routine));
    break;
  case 7: // ...
    eh->tryExternal(F90MODPROC(faulty,protected_routine));
    break;
  case 8: // ...
    eh->tryExternal(F90MODPROC(faulty,unprotected_routine));
    break;
  case 9:
    try {eh->tryExternal(F90MODPROC(faulty,error_routine));}
    catch(SasException::Exception e) {
      std::cerr << "...Caught error>>> " << e.message << std::endl;
      try {eh->tryExternal(F90MODPROC(faulty,fatal_routine));}
      catch(SasException::Exception e) {
	std::cerr << "...Caught error>>> " << e.message << std::endl;
	try{eh->tryExternal(F90MODPROC(faulty,unprotected_routine));}
	catch(SasException::Exception e) {
	std::cerr << "...Caught error>>> " << e.message << std::endl;
	}
      }
    }
    break;
  }
}
    

int main(int argc, char** argv)
{
  if (argc != 2 ) return 5;
  int item=0;item=atoi(argv[1]);

//  ExceptionErrorHandler local;
//  ErrorHandler* oldHandler = errHandler.handler(&local);
  bool UnknownError=false;

  {
      SetExceptionErrorHandler eeh;
      
      errHandler.name("ThisTestProgram");
      

      try { // piece of code, that may bail out
	  handle(item);
      }
      catch (SasException::Error e) {
	  std::cerr << "Error '" << e.code << "' in " << e.name << " : " 
		    << e.message << std::endl;
	  return 2;
      }
      catch (SasException::Fatal e) {
	  std::cerr << "Panic '" << e.code << "' in " << e.name << " : " 
		    << e.message << std::endl;
	  return 3;
      }
      catch (SasException::Exception e) { // last SAS resort
	  // Sas library threw an exception,
	  // but the implementor did not catch it!
	  std::cerr << "SasException: " << e.message << std::endl;
	  return 4;
      }
      catch (...) { // LAST RESORT!
	  UnknownError=true;
      }

//  errHandler.handler(oldHandler); // return to default error handler
  }

  if (UnknownError) {
    errstr << "Fatal exception caught, please report bug"
	   << Sas::fatal("UnknownException");
  }
  return 0;
}
    
